@@ -48,7 +48,8 @@ from .Dialogs import (DistUpgradeDialog,
                       NoUpdatesDialog,
                       StoppedUpdatesDialog,
                       PartialUpgradeDialog,
-                      UnsupportedDialog)
+                      UnsupportedDialog,
+                      UpdateErrorDialog)
 from .InstallProgress import InstallProgress
 from .MetaReleaseGObject import MetaRelease
 from .UpdateProgress import UpdateProgress
@@ -83,7 +84,11 @@ class UpdateManager(Gtk.Window):
         self.set_icon_name("system-software-update")
         self.set_resizable(False)
         self.set_position(Gtk.WindowPosition.CENTER)
-        self.set_size_request(500, -1)
+
+        # Keep window at a constant size
+        ctx = self.get_style_context()
+        ctx.connect("changed", lambda ctx: self.resize_to_standard_width())
+        self.resize_to_standard_width()
 
         # Signals
         self.connect("delete-event", self.close)
@@ -105,6 +110,18 @@ class UpdateManager(Gtk.Window):
             self.options and self.options.devel_release,
             self.options and self.options.use_proposed)
 
+    def resize_to_standard_width(self):
+        if self.get_resizable():
+            return  # only size to a specific em if we are a static size
+        num_em = 33  # per SoftwareUpdates spec
+        dpi = self.get_screen().get_resolution()
+        if dpi <= 0:
+            dpi = 96
+        ctx = self.get_style_context()
+        size = ctx.get_property("font-size", Gtk.StateFlags.NORMAL)
+        width = dpi / 72 * size * num_em
+        self.set_size_request(width, -1)
+
     def on_initial_focus_in(self, widget, event):
         """callback run on initial focus-in (if started unmapped)"""
         self.unstick()
@@ -129,6 +146,7 @@ class UpdateManager(Gtk.Window):
 
         # Reset state
         self.set_resizable(False)
+        self.resize_to_standard_width()
         self.set_sensitive(True)
         if self.get_window() is not None:
             self.get_window().set_cursor(None)
@@ -163,17 +181,17 @@ class UpdateManager(Gtk.Window):
 
         self._start_pane(UpdateProgress(self))
 
-    def start_available(self, cancelled_update=False):
+    def start_available(self, cancelled_update=False, error_occurred=False):
         self._look_busy()
         self.refresh_cache()
 
         pane = self._make_available_pane(self.cache.install_count,
                                          os.path.exists(REBOOT_REQUIRED_FILE),
-                                         cancelled_update)
+                                         cancelled_update, error_occurred)
         self._start_pane(pane)
 
-    def _make_available_pane(self, install_count, need_reboot,
-                             cancelled_update):
+    def _make_available_pane(self, install_count, need_reboot=False,
+                             cancelled_update=False, error_occurred=False):
         if install_count == 0:
             # Need Restart > New Release > No Updates
             if need_reboot:
@@ -184,11 +202,13 @@ class UpdateManager(Gtk.Window):
             elif cancelled_update:
                 return StoppedUpdatesDialog(self)
             else:
-                return NoUpdatesDialog(self)
+                return NoUpdatesDialog(self, error_occurred=error_occurred)
         else:
             header = None
             desc = None
-            if cancelled_update:
+            if error_occurred:
+                desc = _("Some software couldn’t be checked for updates.")
+            elif cancelled_update:
                 header = _("You stopped the check for updates.")
                 desc = _("Updated software is available from "
                          "a previous check.")
@@ -197,8 +217,11 @@ class UpdateManager(Gtk.Window):
     def start_install(self):
         self._start_pane(InstallProgress(self))
 
-    def start_error(self, header, desc):
-        self._start_pane(ErrorDialog(self, header, desc))
+    def start_error(self, is_update_error, header, desc):
+        if is_update_error:
+            self._start_pane(UpdateErrorDialog(self, header, desc))
+        else:
+            self._start_pane(ErrorDialog(self, header, desc))
 
     def _start_pane(self, pane):
         self._look_busy()
@@ -271,7 +294,7 @@ class UpdateManager(Gtk.Window):
                      "Please use the package manager \"Synaptic\" or run "
                      "\"sudo apt-get install -f\" in a terminal to fix "
                      "this issue at first.")
-            self.start_error(header, desc)
+            self.start_error(True, header, desc)
         except SystemError as e:
             header = _("Could not initialize the package information")
             desc = _("An unresolvable problem occurred while "
@@ -279,7 +302,7 @@ class UpdateManager(Gtk.Window):
                      "Please report this bug against the 'update-manager' "
                      "package and include the following error "
                      "message:\n") + str(e)
-            self.start_error(header, desc)
+            self.start_error(True, header, desc)
 
         # Let the Gtk event loop breath if it hasn't had a chance.
         while Gtk.events_pending():
@@ -295,9 +318,7 @@ class UpdateManager(Gtk.Window):
                      "Please report this bug against the 'update-manager' "
                      "package and include the following error "
                      "message:\n") + str(e)
-            self.start_error(header, desc)
-
-        self.unity.set_updates_count(self.cache.install_count)
+            self.start_error(True, header, desc)
 
         if self.update_list.distUpgradeWouldDelete > 0:
             self._start_pane(PartialUpgradeDialog(self))
